/*
 * CartesianProduct.h
 *
 *  Created on: Jan 30, 2015
 *      Author: Andre Dozier
 */

#ifndef CARTESIANPRODUCT_H_
#define CARTESIANPRODUCT_H_

#include <vector>
#include <stdlib.h>
#include <stdexcept>
#include <iostream>
#include "DyproUtils.h"
#include "rapidxml.hpp"
using namespace std;
using namespace rapidxml;

/**
 * A low memory Cartesian product calculator for assessing combinatorial
 * (discrete problems).
 */
class CartesianProduct {
private:
	// private instance variables
	vector<long long> _cumCardinality;
	long long _cardinality;
	vector<long long> _tmpIndices;

	// private member functions
	void _updateCardinality();

public:
	// public instance variables
	int ndim;
	vector<vector<double> > sets;

	// constructors
	CartesianProduct();
	CartesianProduct(vector<vector<double> >&);

	// properties
	long long getCardinality(void);
	void getIndices(vector<long long>&, long long);
	long long getFlatIndex(vector<long long>&);

	// member functions
	void addSets(vector<vector<double> >&);
	void addSet(vector<double>&);
	void getElement(vector<double>&, vector<long long>&);
	void getElement(vector<double>&, long long);

	// static member functions
	static CartesianProduct fromXML(xml_node<> *node);
};

#endif /* CARTESIANPRODUCT_H_ */
