/*
 * Constraint.h
 *
 *  Created on: Jul 8, 2015
 *      Author: Andre Dozier
 */

#ifndef CONSTRAINT_H_
#define CONSTRAINT_H_

#include <vector>
#include <stdexcept>
#include <iostream>
using namespace std;

enum ConstraintType {
	LT, LE, EQ, GE, GT
};

class Constraint {

	ConstraintType _type;
	CartesianProduct* _self;
	CartesianProduct* _other;
	int _selfVar, _otherVar;
	bool _selfIsState, _otherIsState;
	vector<double> _selfElem, _otherElem;

public:

	Constraint(CartesianProduct& self, int selfVar, bool selfIsState, ConstraintType type,
			CartesianProduct& other, int otherVar, bool otherIsState) {
		_type = type;
		_self = &self;
		_other = &other;
		_selfVar = selfVar;
		_otherVar = otherVar;
		_selfIsState = selfIsState;
		_otherIsState = otherIsState;
		_selfElem.resize(self.ndim, 0.0);
		_otherElem.resize(other.ndim, 0.0);
	}

	bool satisfied(long long state_i, long long dec_i) {

		_self->getElement(_selfElem, _selfIsState ? state_i : dec_i);
		_other->getElement(_otherElem, _otherIsState ? state_i : dec_i);

		switch (_type) {

		case LT:
			return _selfElem[_selfVar] < _otherElem[_otherVar];

		case LE:
			return _selfElem[_selfVar] <= _otherElem[_otherVar];

		case EQ:
			return _selfElem[_selfVar] == _otherElem[_otherVar];

		case GE:
			return _selfElem[_selfVar] >= _otherElem[_otherVar];

		case GT:
			return _selfElem[_selfVar] > _otherElem[_otherVar];

		default:
			throw runtime_error("A ConstraintType is not implemented!");
		}

	}

};

#endif /* CONSTRAINT_H_ */
