/*
 * ConstraintParser.h
 *
 *  Created on: Jul 8, 2015
 *      Author: Andre Dozier
 */

#ifndef CONSTRAINTPARSER_H_
#define CONSTRAINTPARSER_H_

#include <string>
#include <vector>
#include <stdexcept>
using namespace std;

#include "rapidxml.hpp"
using namespace rapidxml;

#include "Constraint.h"

class ConstraintParser {

	vector<CartesianProduct *> _selfs;
	vector<int> _selfIndex;
	vector<int> _otherIndex;
	vector<ConstraintType> _types;

public:

	vector<bool> otherIsState;

	ConstraintParser() {
	}

	void add(CartesianProduct &grid, int dim, xml_node<> *node) {
		xml_attribute<> *attr;
		int *i;
		ConstraintType * t;

		// less than
		attr = node->first_attribute("lt", 0, false);
		if (!attr)
			attr = node->first_attribute("lessthan", 0, false);
		if (attr)
			add(grid, dim, attr->value(), LT);

		// less than or equal to
		attr = node->first_attribute("le", 0, false);
		if (!attr)
			attr = node->first_attribute("lessthanorequalto", 0, false);
		if (attr)
			add(grid, dim, attr->value(), LE);

		// equal to
		attr = node->first_attribute("eq", 0, false);
		if (!attr)
			attr = node->first_attribute("equalto", 0, false);
		if (attr)
			add(grid, dim, attr->value(), EQ);

		// greater than or equal to
		attr = node->first_attribute("ge", 0, false);
		if (!attr)
			attr = node->first_attribute("greaterthanorequalto", 0, false);
		if (attr)
			add(grid, dim, attr->value(), GE);

		// greater than
		attr = node->first_attribute("gt", 0, false);
		if (!attr)
			attr = node->first_attribute("greaterthan", 0, false);
		if (attr)
			add(grid, dim, attr->value(), GT);

	}

	void add(CartesianProduct &grid, int dim, char *val, ConstraintType type) {

		switch (val[0]) {

		case 'D':
		case 'd':
			otherIsState.push_back(false);
			break;

		case 'S':
		case 's':
			otherIsState.push_back(true);
			break;

		default:
			throw runtime_error(
					"Cannot parse the constraint, must start with either 'd' (for decision variable) or 's' (for state variable).");
		}

		const char * v = (const char *) (val + 1);
		int other = toInt(v);
		_selfIndex.push_back(dim);
		_otherIndex.push_back(other);
		_types.push_back(type);
		_selfs.push_back(&grid);

	}

	Constraints getConstraints(LinearGrid& states, LinearGrid& decisions) {

		int i;
		int j;
		Constraints retVal;
		for (i = 0; i < _selfs.size(); i++) {

			if (_selfs[i] == &states) {
				if (otherIsState[i]) {
					Constraint c0(states, _selfIndex[i], true, _types[i],
							states, _otherIndex[i], true);
					retVal.add(c0, otherIsState[i]);
				} else {
					Constraint c1(states, _selfIndex[i], true, _types[i],
							decisions, _otherIndex[i], false);
					retVal.add(c1, otherIsState[i]);
				}
			}

			if (_selfs[i] == &decisions) {
				if (otherIsState[i]) {
					Constraint c2(decisions, _selfIndex[i], false, _types[i],
							states, _otherIndex[i], true);
					retVal.add(c2, otherIsState[i]);
				} else {
					Constraint c3(decisions, _selfIndex[i], false, _types[i],
							decisions, _otherIndex[i], false);
					retVal.add(c3, otherIsState[i]);
				}
			}

		}
		return retVal;
	}

};

#endif /* CONSTRAINTPARSER_H_ */
