/*
 * DyproSolution.h
 *
 *  Created on: Feb 9, 2015
 *      Author: Andre Dozier
 */

#ifndef DYPROSOLUTION_H_
#define DYPROSOLUTION_H_

#include <vector>
#include <cmath>
#include <stdexcept>
#include <iostream>
#include "Bounds.h"
#include "LinearGrids.h"
#include "DyproSolutionPoint.h"
#include "DyproSolutionAtTime.h"
using namespace std;

class DyproSolution {

private:

	// resize
	int _nT, _nR, _init;
	LinearGrids *_states, *_decisions;

public:

	// public instance variables
	vector<DyproSolutionAtTime> solution;

	// constructor
	DyproSolution();
	void resize(int numTimesteps,
			LinearGrids& states, LinearGrids& decisions, int numRndVarDims,
			double initValue);

	// decisions
	DyproSolutionPoint& getSolutionPoint(int, long long);

	// stages
	DyproSolutionAtTime& operator[](int);

	// bounds
	Bounds getDecisionBounds();
	Bounds getStateBounds();

	// error
	double maxError();

	// print
	void save(ostream& o, int maxPrints = 0);
	void load(istream& fs);

};

#endif /* DYPROSOLUTION_H_ */
