/*
 * DyproTrace.h
 *
 *  Created on: Feb 9, 2015
 *      Author: Andre Dozier
 */

#ifndef DYPROTRACE_H_
#define DYPROTRACE_H_

#include <vector>
#include <iostream>
#include <iomanip>
#include "LinearGrids.h"
#include "DyproSolution.h"
#include "SequentialDecisionProcess.h"
#include "RandomTimeseries.h"
#include "ObjectiveAccumulation.h"
using namespace std;

class DyproTrace {

public:

	// instance variables
	RandomTimeseries rndVars;
	vector<DyproSolutionPoint> values;
	vector<string> extraHeaders;
	vector<vector<double> > extras;
	vector<bool> extraCalcPV;
	unsigned int nTimesteps;
	unsigned int nRndVars;
	unsigned int nExtraVars;

	// resize
	void resize(unsigned int numTimesteps, LinearGrids& states,
			LinearGrids& decisions, unsigned int numRndVars,
			unsigned int numExtraVars);

	// [] operator
	DyproSolutionPoint& operator[](unsigned int t);

	// get total value
	double totalValue(ObjectiveAccumulator *objAccum,
			double salvageValue, double discount);
	double extraTotalValue(ObjectiveAccumulator *objAccum,
			double salvageValue, double discount, int dim);

	// extras
	void saveExtraHeaders(ostream &o);
	void saveExtras(ostream &o, unsigned int t);
	void saveExtraNext(ostream &o, ObjectiveAccumulator *objAccum,
			double salvageValue, double discount);
};

#endif /* DYPROTRACE_H_ */
