/*
 * ObjectiveAccumulation.h
 *
 *  Created on: Feb 6, 2015
 *      Author: Andre Dozier
 */

#ifndef OBJECTIVEACCUMULATION_H_
#define OBJECTIVEACCUMULATION_H_

#include <map>
#include <string>
using namespace std;

enum ObjectiveAccumulation {
	Additive = 1, Multiplicative = 2, MaxStage = 3, MinStage = 4
};

static map<string, ObjectiveAccumulation> objAccumulationFromString = { {
		"additive", Additive }, { "multiplicative", Multiplicative }, {
		"maxstage", MaxStage }, { "minstage", MinStage } };

#define takeMin(a,b) ({ __typeof__ (a) _a = (a); \
					__typeof__ (b) _b = (b); \
		   	   	    _a < _b ? _a : _b; })

#define takeMax(a,b) ({ __typeof__ (a) _a = (a); \
					__typeof__ (b) _b = (b); \
					_a > _b ? _a : _b; })

// general interface (& base class)
class ObjectiveAccumulator {
public:
	virtual ~ObjectiveAccumulator() {
	}
	virtual double reduce(double currValue, double nextValue,
			double discount) = 0;
	static ObjectiveAccumulator* select(ObjectiveAccumulation);
	ObjectiveAccumulator* operator =(ObjectiveAccumulation a) {
		return select(a);
	}
};

// additive
class AdditiveAccumulator: public ObjectiveAccumulator {
public:
	double reduce(double, double, double);
};

// multiplicative
class MultiplicativeAccumulator: public ObjectiveAccumulator {
public:
	double reduce(double, double, double);
};

// max stage
class MaximizeStageAccumulator: public ObjectiveAccumulator {
public:
	double reduce(double, double, double);
};

// min stage
class MinimizeStageAccumulator: public ObjectiveAccumulator {
public:
	double reduce(double, double, double);
};

#endif /* OBJECTIVEACCUMULATION_H_ */
