/*
 * ObjectiveTie.h
 *
 *  Created on: Feb 6, 2015
 *      Author: Andre Dozier
 */

#ifndef OBJECTIVETIE_H_
#define OBJECTIVETIE_H_

#include <numeric>
#include <stdexcept>
#include <map>
#include "DyproSolutionPoint.h"

using namespace std;

enum ObjectiveTie {
	First = 1,
	Last = 2,
	HighDecision = 3,
	LowDecision = 4,
	HighState = 5,
	LowState = 6
};

static map<string, ObjectiveTie> objTieFromString = { { "first", First }, {
		"last", Last }, { "highdecision", HighDecision }, { "lowdecision",
		LowDecision }, { "highstate", HighState }, { "lowstate", LowState } };

const double ZERO_D = 0.0;

class ObjectiveTieBreaker {
public:
	virtual ~ObjectiveTieBreaker() {
	}
	virtual void breakTie(DyproSolutionPoint&, DyproSolutionPoint&, bool*) = 0;
	static ObjectiveTieBreaker* select(ObjectiveTie);
	ObjectiveTieBreaker* operator =(ObjectiveTie a) {
		return select(a);
	}
};

class TieBreakerFirst: public ObjectiveTieBreaker {
public:
	void breakTie(DyproSolutionPoint&, DyproSolutionPoint&, bool*);
};

class TieBreakerLast: public ObjectiveTieBreaker {
public:
	void breakTie(DyproSolutionPoint&, DyproSolutionPoint&, bool*);
};

class TieBreakerHighDecision: public ObjectiveTieBreaker {
public:
	void breakTie(DyproSolutionPoint&, DyproSolutionPoint&, bool*);
};

class TieBreakerLowDecision: public ObjectiveTieBreaker {
public:
	void breakTie(DyproSolutionPoint&, DyproSolutionPoint&, bool*);
};

class TieBreakerHighState: public ObjectiveTieBreaker {
public:
	void breakTie(DyproSolutionPoint&, DyproSolutionPoint&, bool*);
};

class TieBreakerLowState: public ObjectiveTieBreaker {
public:
	void breakTie(DyproSolutionPoint&, DyproSolutionPoint&, bool*);
};

#endif /* OBJECTIVETIE_H_ */
