/*
 * RandomTimeseries.h
 *
 *  Created on: Feb 12, 2015
 *      Author: Andre Dozier
 */

#ifndef RANDOMTIMESERIES_H_
#define RANDOMTIMESERIES_H_

#include <vector>
#include <cstdint>
#include <stdexcept>
#include <random>
#include <cstdlib>
#include <time.h>
#include <iostream>
#include "ProbabilityWeightedSet.h"
#include "rapidxml.hpp"
#include "DyproUtils.h"
using namespace std;
using namespace rapidxml;

class RandomTimeseries {

private:
	// generate timeseries
	mt19937 _gen;
	uniform_real_distribution<double> _dist;

	// fixed timeseries
	bool _fixed;
	Timeseries _ts;
	unsigned int _seed;
	vector<double> _genValues;

public:

	// specify type
	bool isStochastic;

	// constructor
	RandomTimeseries();

	// properties
	long long size();
	vector<double>& operator[](int t);
	double getGeneratedValue(int t);

	// member functions
	void setTimeseries(Timeseries ts);
	void resize(int nTimesteps, int nRandomVars);
	void seed(unsigned int seed = 0);
	void reseed();
	void generate(vector<ProbabilityWeightedSet>& pset, int nSolutionTimesteps);

	// static from string
	static RandomTimeseries fromXML(xml_node<> *node, int nTimesteps,
			int nRandomVars, bool seed);
};

#endif /* RANDOMTIMESERIES_H_ */
