/*
 * SequentialDecisionProcess.h
 *
 *  Created on: Feb 2, 2015
 *      Author: Andre Dozier
 */

#ifndef SEQUENTIALDECISIONPROCESS_H_
#define SEQUENTIALDECISIONPROCESS_H_

#include <vector>
#include "DyproSolutionPoint.h"
#include "Bounds.h"
using namespace std;

class SequentialDecisionProcess {

public:
	// current state and decision variables
	/**
	 * Contains information on the current (not optimal) solution such as
	 * decisions, states, and calculated value
	 */
	LinearGrids states;
	LinearGrids decisions;
	DyproSolutionPoint curr;
	bool tracing;

	// default destructor and constructor
	SequentialDecisionProcess() {
		tracing = false;
	}

	virtual ~SequentialDecisionProcess() {
	}

	// abstract class functions...

	/**
	 * A function that simulates the system and updates the next state and
	 * decision variables to remain feasible. Returns true if feasible. False if not.
	 */
	virtual bool simulate(vector<double>& nextState, Bounds& stateBounds, Bounds& decisionBounds) = 0;

	/**
	 * Calculates the objective function value based on the current decisions
	 * and states
	 */
	virtual double value() = 0;

	/**
	 * An array with extra variable header names to be saved when tracing
	 * a solution
	 */
	virtual vector<string> extraHeaders() {
		return {};
	}

	/**
	 * An array of extra variables to be saved when tracing a solution
	 */
	virtual vector<double> extra() {
		return {};
	}

	/**
	 * An array that specifies which variables should be totaled (true) and
	 * those that should not be totaled (false)
	 */
	virtual vector<bool> extraCalcPV() {
		return {};
	}

};

#endif /* SEQUENTIALDECISIONPROCESS_H_ */
